﻿/* 
    Copyright (c) Microsoft Corporation. All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code.  For the terms of the license, please see the 
    license agreement between you and Microsoft.
  
    To see all code Samples for Windows Store apps and Windows Phone Store apps, visit http://code.msdn.microsoft.com/windowsapps
  
*/

using Bookstore3WPSL8.Resources;
using Microsoft.Phone.Controls;
using System;

namespace Bookstore3WPSL8
{
	public partial class MainPage : PhoneApplicationPage
	{
		// Constructor
		public MainPage()
		{
			InitializeComponent();

			this.InitializeApplicationBar();
		}

		// The application bar controls are not bound to viewmodel properties, so imperatively set their text to localizable strings.
		private void InitializeApplicationBar()
		{
			this.BookstoreViewModel.AtticusCommand.CanExecuteChanged += MainViewModelAtticusCommand_CanExecuteChanged;
			this.BookstoreViewModel.MobyDickCommand.CanExecuteChanged += MainViewModelMobyDickCommand_CanExecuteChanged;
			// Explicit initial call to the event handlers to initialize the enabled state of the application bar controls.
			this.MainViewModelMobyDickCommand_CanExecuteChanged(this, EventArgs.Empty);
			this.MainViewModelAtticusCommand_CanExecuteChanged(this, EventArgs.Empty);

			if (this.ApplicationBar != null && this.ApplicationBar.Buttons.Count != 0 && this.ApplicationBar.MenuItems.Count != 0)
			{
				(this.ApplicationBar.Buttons[0] as Microsoft.Phone.Shell.ApplicationBarIconButton).Text = AppResources.MobyStringShort;
				(this.ApplicationBar.Buttons[1] as Microsoft.Phone.Shell.ApplicationBarIconButton).Text = AppResources.AtticusStringShort;
				(this.ApplicationBar.MenuItems[0] as Microsoft.Phone.Shell.ApplicationBarMenuItem).Text = AppResources.MobyStringLong;
				(this.ApplicationBar.MenuItems[1] as Microsoft.Phone.Shell.ApplicationBarMenuItem).Text = AppResources.AtticusStringLong;
			}
		}

		// Handle the viewmodel AtticusCommand's CanExecuteChanged event to update the enabled state of the application bar controls, which cannot be bound to the command.
		void MainViewModelMobyDickCommand_CanExecuteChanged(object sender, EventArgs e)
		{
			if (this.ApplicationBar != null && this.ApplicationBar.Buttons.Count != 0 && this.ApplicationBar.MenuItems.Count != 0)
			{
				(this.ApplicationBar.Buttons[0] as Microsoft.Phone.Shell.ApplicationBarIconButton).IsEnabled = this.BookstoreViewModel.MobyDickCommand.CanExecute(null);
				(this.ApplicationBar.MenuItems[0] as Microsoft.Phone.Shell.ApplicationBarMenuItem).IsEnabled = this.BookstoreViewModel.MobyDickCommand.CanExecute(null);
			}
		}

		// Handle the viewmodel AtticusCommand's CanExecuteChanged event to update the enabled state of the application bar controls, which cannot be bound to the command.
		void MainViewModelAtticusCommand_CanExecuteChanged(object sender, EventArgs e)
		{
			if (this.ApplicationBar != null && this.ApplicationBar.Buttons.Count != 0 && this.ApplicationBar.MenuItems.Count != 0)
			{
				(this.ApplicationBar.Buttons[1] as Microsoft.Phone.Shell.ApplicationBarIconButton).IsEnabled = this.BookstoreViewModel.AtticusCommand.CanExecute(null);
				(this.ApplicationBar.MenuItems[1] as Microsoft.Phone.Shell.ApplicationBarMenuItem).IsEnabled = this.BookstoreViewModel.AtticusCommand.CanExecute(null);
			}
		}

		// Although the application bar controls are not bound to the viewmodel AtticusCommand, we can still execute it via an event handler.
		private void ApplicationBar_MobyDick_Click(object sender, EventArgs e)
		{
			// A command takes a parameter and in this case we can pass null.
			this.BookstoreViewModel.MobyDickCommand.Execute(null);
		}

		// Although the application bar controls are not bound to the viewmodel MobyDickCommand, we can still execute it via an event handler.
		private void ApplicationBar_Atticus_Click(object sender, EventArgs e)
		{
			// A command takes a parameter and in this case we can pass null.
			this.BookstoreViewModel.AtticusCommand.Execute(null);
		}

		// A convenient helper property to access a typecast reference to the DataContext. The DataContext is set in markup to an instance
		// of BookstoreViewModel, and this cast is correct as long as that markup remains intact.
		private BookstoreViewModel BookstoreViewModel
		{
			get
			{
				return this.DataContext as BookstoreViewModel;
			}
		}
	}
}